/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.lidar.amapvox.visualization.util.View;
import java.awt.BorderLayout;
import org.jdesktop.swingx.JXTitledPanel;
import fr.amap.lidar.amapvox.visualization.controllers.ViewController;
import fr.amap.lidar.amapvox.visualization.opengl.windows.GLNewtCanvas;
import java.awt.Color;
import java.awt.LinearGradientPaint;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import org.jdesktop.swingx.JXLabel;
import org.jdesktop.swingx.JXPanel;
import org.jdesktop.swingx.painter.MattePainter;

/**
 * View where OpenGL drawing functions are performed.
 */
public class ViewView extends View {

    private final ViewController controller;

    private JXTitledPanel titledPanel;

    private GLNewtCanvas canvas;

    private JXPanel gradientPanel;
    private JXLabel gradientMinLabel;
    private JXLabel gradientMediumLabel;
    private JXLabel gradientMaxLabel;
    private LinearGradientPaint gradientPaint;

    /**
     * Creates new form ViewFrame
     * @throws java.lang.Exception
     */
    public ViewView() throws Exception {
        super();
        this.controller = new ViewController();
        controller.setCanvas(canvas);
    }

    public final ViewController getController() {
        return controller;
    }

    @Override
    protected void initContainer() throws Exception {
        setLocationRelativeTo(getParent());
        setLayout(new BorderLayout());

        getContentPane().addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                updateGradient();
            }
        });
    }

    public final void updateGradient() {
        final Color[] colors = controller.getCurrentColorBar();

        final float[] fractions = new float[colors.length];
        for (int i = 0; i < fractions.length; i++) {
            fractions[i] = 1f / (float) fractions.length * (float) i;
        }
        
        final float minColor = (float) (colors[0].getRed() + 
                colors[0].getGreen() + colors[0].getBlue()) / 3f / 255;
        gradientMinLabel.setForeground(minColor < 0.5f ? Color.WHITE : Color.BLACK);
        
        final int mediumID = Math.round((float)(colors.length - 1) / 2);
        final float mediumColor = (float) (colors[mediumID]
                .getRed() + colors[mediumID].getGreen() + colors[mediumID].getBlue()) / 3f / 255;
        gradientMaxLabel.setForeground(mediumColor < 0.5f ? Color.WHITE : Color.BLACK);
        
        final int maxID = colors.length - 1;
        final float maxColor = (float) (colors[maxID].getRed() 
                + colors[maxID].getGreen() + colors[maxID].getBlue()) / 3f / 255;
        gradientMaxLabel.setForeground(maxColor < 0.5f ? Color.WHITE : Color.BLACK);

        gradientPaint = new LinearGradientPaint(0, 0,
                getContentPane().getWidth(), 0, fractions, colors);
        gradientPanel.setBackgroundPainter(new MattePainter(gradientPaint));
    }
    
    public final void setGradientTexts(final String min, final String medium, final String max) {
        gradientMinLabel.setText(min);
        gradientMediumLabel.setText(medium);
        gradientMaxLabel.setText(max);
    }

    @Override
    protected void initComponents() throws Exception {
        titledPanel = new JXTitledPanel();
        addLocaleCallback((locale) -> {
            titledPanel.setTitle(bundle.getString("viewView_Title"));
        });
        titledPanel.getContentContainer().setLayout(new BorderLayout());

        canvas = new GLNewtCanvas();
        titledPanel.getContentContainer().add(canvas, BorderLayout.CENTER);

        gradientPanel = new JXPanel();
        gradientPanel.setLayout(new BorderLayout());
        titledPanel.getContentContainer().add(gradientPanel, BorderLayout.SOUTH);

        gradientMinLabel = new JXLabel("0%");
        gradientPanel.add(gradientMinLabel, BorderLayout.WEST);
        gradientMediumLabel = new JXLabel("50%");
        gradientMediumLabel.setTextAlignment(JXLabel.TextAlignment.CENTER);
        gradientMediumLabel.setHorizontalAlignment(JXLabel.CENTER);
        gradientPanel.add(gradientMediumLabel, BorderLayout.CENTER);
        gradientMaxLabel = new JXLabel("100%");
        gradientPanel.add(gradientMaxLabel, BorderLayout.EAST);

        add(titledPanel, BorderLayout.CENTER);
    }

}
